import requests

from .request_types import *
from requests.auth import HTTPBasicAuth


class SignicatAPI:
    PDF_DOCUMENT_TYPE = 'application/pdf'

    def __init__(self, url, basic_auth_user: str, basic_auth_password: str):
        self.headers = {
            'Content-Type': 'application/json'
        }
        self.url = url
        self.access_token = None
        self.basic_auth_user = basic_auth_user
        self.basic_auth_password = basic_auth_password

    def __url(self, url):
        return self.url + url

    def __authorize(self):
        """
        Perform authentication call to create access token.
        Token will be stored in access_token property.
        """
        r = requests.post(self.__url('/oidc/token'),
                          auth=HTTPBasicAuth(self.basic_auth_user, self.basic_auth_password),
                          headers={
                              'Content-Type': 'application/x-www-form-urlencoded'
                          },
                          data={
                              'scope': 'client.signature',
                              'grant_type': 'client_credentials'
                          })

        self.access_token = r.json()['access_token']

    def __to_json(self, response):
        return response.json()

    @staticmethod
    def from_json(data, type):
        return json.loads(data, object_hook=lambda d: type(**d))

    def __set_headers(self, need_auth=True):
        if need_auth:
            self.headers['Authorization'] = 'Bearer {0}'.format(self.access_token)

    def __get(self, url, need_auth=True):
        self.__authorize()
        self.__set_headers(need_auth)

        return requests.get(self.__url(url), headers=self.headers)

    def __post(self, url, data=None, need_auth=True):
        self.__authorize()
        self.__set_headers(need_auth)

        return requests.post(self.__url(url), headers=self.headers, data=data)

    def post_upload_one_document(self, document_file: bytes, document_content_type: str = PDF_DOCUMENT_TYPE):
        """
        Upload one document to receive documentIds
        """
        self.headers['Content-Type'] = document_content_type
        response = self.__post('/sign/documents', data=document_file)
        return response.status_code, self.__to_json(response)

    def post_sign_order(self, sign_order: SignOrder):
        """
        Create sign order
        """
        response = self.__post('/sign/orders', data=sign_order.to_json())
        return response.status_code, self.__to_json(response)

    def get_task_events(self, order_id: str, task_id: str):
        """
        Get all events related to a task
        """
        response = self.__get('/sign/orders/{0}/tasks/{1}/events'.format(order_id, task_id))
        return response.status_code, self.__to_json(response)

    def get_packaging_task_result_document(self, order_id: str, task_id: str):
        """
        Get document related to packaging task by order id
        """
        response = self.__get('/sign/orders/{0}/packaging-tasks/{1}/result'.format(order_id, task_id))
        return response.status_code, self.__to_json(response)

    def get_signing_order(self, order_id: str) -> (int, SignOrder):
        """
        Get the signing order with the given ID
        """
        response = self.__get('/sign/orders/{0}'.format(order_id))

        return response.status_code, self.__to_json(response)

    def get_document(self, document_id: str) -> (int, bytes):
        """
        Download document
        """
        response = self.__get('/sign/documents/{0}'.format(document_id))
        return response.status_code, response.content



import json

SWISHFUND_NO_REPLY_EMAIL = 'no-reply@swishfund.nl'

class RequestType:
    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True)


class Document:

    def __init__(self, id: str, action: str, description: str, document_ref: str, source: str):
        self.source = source
        self.id = id
        self.documentRef = document_ref
        self.description = description
        self.action = action


class SignatureMethod:
    def __init__(self, handwritten: bool, name: str, type: str):
        self.type = type
        self.name = name
        self.handwritten = handwritten

    @staticmethod
    def sms(handwritten: bool = True):
        return SignatureMethod(handwritten, 'scid-sms', 'AUTHENTICATION_BASED')

    @staticmethod
    def idin(handwritten: bool = True):
        return SignatureMethod(handwritten, 'idin', 'AUTHENTICATION_BASED')


class NotificationSchedule:
    def __init__(self, trigger_status: str):
        self.triggerStatus = trigger_status


class Notification:
    def __init__(self, id: str, header: str, message: str, recipient: str, schedule: [NotificationSchedule], sender: str, type: str):
        self.type = type
        self.sender = sender
        self.schedule = schedule
        self.recipient = recipient
        self.message = message
        self.header = header
        self.id = id

    @staticmethod
    def created(task_notification_id: str, to_email: str):
        """
        Default created notification to reduces boilerplate code
        """
        return Notification(task_notification_id,
                                     'Signicat - document ready to be signed',
                                     'Dear,\n\nA document is ready to be signed at the following link:\n\n${signCodeUrl}\n\nRegards,\nSignicat',
                                     to_email,
                                     [NotificationSchedule('CREATED')],
                                     SWISHFUND_NO_REPLY_EMAIL,
                                     'EMAIL')

    @staticmethod
    def completed(task_notification_id: str, to_email: str):
        """
        Default completed notification to reduces boilerplate code
        """
        return Notification(task_notification_id,
                                     'Signicat - document signed successfully',
                                     'Dear,\n\nYou have successfully signed the document\n\nRegards,\nSignicat',
                                     to_email,
                                     [NotificationSchedule('COMPLETED')],
                                     SWISHFUND_NO_REPLY_EMAIL,
                                     'EMAIL')

    @staticmethod
    def pades(task_notification_id: str, to_email: str):
        """
        Default pades notification to reduces boilerplate code
        """
        return Notification(task_notification_id,
                     'Signicat - pades generated successfully',
                     'Dear,\n\nThe pades has been generated successfully\n\nRegards,\nSignicat',
                     to_email,
                     [NotificationSchedule('COMPLETED')],
                     SWISHFUND_NO_REPLY_EMAIL,
                     'EMAIL')


class Task:
    def __init__(self, id: str, configuration: str, documents: [Document], signature_methods: [SignatureMethod], on_task_complete: str, notifications: [Notification]):
        self.id = id
        self.configuration = configuration
        self.documents = documents
        self.signatureMethods = signature_methods
        self.notifications = notifications
        self.onTaskComplete = on_task_complete


class PackagingTaskDocuments:
    def __init__(self, task_id: str, document_ids: [str]):
        self.documentIds = document_ids
        self.taskId = task_id


class PackagingTask:
    def __init__(self, id: str, send_to_archive: bool, method: str, notifications: [Notification], documents: [PackagingTaskDocuments]):
        self.documents = documents
        self.notifications = notifications
        self.method = method
        self.sendToArchive = send_to_archive
        self.id = id


class SignOrder(RequestType):
    def __init__(self, client_reference: str or None, tasks: [Task], packaging_tasks: [PackagingTask]):
        self.clientReference = client_reference
        self.tasks = tasks
        self.packagingTasks = packaging_tasks


from distutils.core import setup

setup(
    name='signicat_api',
    packages=['signicat_api'],
    version='1.2.0',
    description='Rabobank Signicat API wrapper',
    author='Theo Bouwman',
    author_email='theo.bouwman@swishfund.nl',
    package_dir={'': 'src'},
    install_requires=[
        'requests==2.21.0'
    ],
)